'''
Cheetah is an open source template engine and code generation tool.

It can be used standalone or combined with other tools and frameworks. Web
development is its principle use, but Cheetah is very flexible and is also
being used to generate C++ game code, Java, sql, form emails and even Python
code.

Homepage, issue tracker
    https://github.com/CheetahTemplate3/cheetah3

Documentation
    https://cheetahtemplate.org/
'''

from Cheetah.Version import *  # noqa
